#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging

class TestCase(object):

    def __init__(self, *args, **kwargs):
        object.__init__(self)
        self.runner = kwargs.get("runner", None)
        self.loader = kwargs.get("loader", None)
        self.logger = kwargs.get("logger", logging.getLogger("test"))

    def before(self):
        pass

    def after(self):
        pass

    def run_all(self):
        for test in self.tests():
            self.run_test(test)

    def run_test(self, test):
        self.before()
        try:
            test()
        finally:
            self.after()

    def error(self, message):
        if not self.logger: return
        self.logger.error(message)

    def warning(self, message):
        if not self.logger: return
        self.logger.warning(message)

    def info(self, message):
        if not self.logger: return
        self.logger.info(message)

    def debug(self, message):
        if not self.logger: return
        self.logger.debug(message)

    def log_stack(self, method = None):
        pass

    @property
    def tests(self):
        return [getattr(self, name) for name in dir(self) if not name == "tests" and hasattr(getattr(self, name), "test")]



#!/usr/bin/python
# -*- coding: utf-8 -*-

class Part(object):

    def __init__(self, owner):
        self.owner = owner

    def __getattr__(self, name):
        if hasattr(self.owner, name):
            return getattr(self.owner, name)
        raise AttributeError("'%s' not found" % name)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import traceback

import appier

from . import loaders

class Runner(object):

    def __init__(self, loader = None):
        self.loader = loader or self.loader_default
        self.on_finish = []

    def run(self):
        raise appier.NotImplementedError()

    def add_on_finish(self, method):
        self.on_finish.append(method)

    def remove_on_finish(self, method):
        self.on_finish.remove(method)

    def run_on_finish(self):
        for method in self.on_finish:
            method()

    @property
    def test_suite(self):
        return self.loader.test_suite(runner = self)

    @property
    def loader_default(self):
        return None

class ConsoleRunner(Runner):

    def run(self):
        result = True
        try:
            for test_case in self.test_suite:
                for test in test_case.tests:
                    print("Running test: %s" % str(test))
                    try:
                        test_case.run_test(test)
                    except appier.AssertionError as exception:
                        result = False
                        print("There was an error: %s" % str(exception))
                    except Exception as exception:
                        result = False
                        print("There was an exception: %s" % str(exception))
                        print("Exception in user code:")
                        print("-" * 60)
                        traceback.print_exc(file = sys.stdout)
                        print("-" * 60)
        finally:
            self.run_on_finish()

        return result

    @property
    def loader_default(self):
        return loaders.PathLoader(".")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

try: from selenium.webdriver.common.keys import Keys
except ImportError: Keys = None

from .. import parts

class RetailPart(parts.Part):

    def login(self, username, password):
        self.driver.get(self.login_url)

        form = self.driver.find_element_by_css_selector(".form")
        username_input = form.find_element_by_name("username")
        username_input.send_keys(username)
        password_input = form.find_element_by_name("password")
        password_input.send_keys(password)
        password_input.send_keys(Keys.ENTER)

    def login_and_redirect(self, username, password):
        self.login(username, password)

        self.waits.redirected_to(self.base_url)

    @property
    def base_url(self):
        base_url = appier.conf("BASE_URL", "https://ripe-retail-test.platforme.com")
        return appier.conf("RETAIL_URL", base_url)

    @property
    def login_url(self):
        return "%s/login" % self.base_url

    @property
    def logout_url(self):
        return "%s/logout" % self.base_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import inspect

import appier

from . import test_cases

class Loader(object):

    def test_suite(self):
        raise appier.NotImplementedError()

class PathLoader(Loader):

    def __init__(self, path = "."):
        self.path = os.path.normpath(os.path.abspath(os.path.expanduser(path)))

    def test_suite(self, **kwargs):
        return [test_cls(loader = self, **kwargs) for test_cls in self._load_classes(self.path)]

    def _load_classes(self, path, recursive = True):
        classes = []
        modules = self._load_modules(path, recursive = recursive)
        for module in modules:
            for name in dir(module):
                value = getattr(module, name)
                if not inspect.isclass(value): continue
                if not issubclass(value, test_cases.TestCase): continue
                classes.append(value)
        return classes

    def _load_packages(self, path, recursive = True):
        packages = []

        if os.path.isdir(path):
            name = os.path.basename(path)
            dir_path = os.path.dirname(path)
            base_name = os.path.splitext(name)[0]
            sys.path.insert(0, dir_path)
            try:
                packages.append(__import__(base_name))
            except ImportError:
                pass
            finally:
                sys.path.remove(dir_path)

        names = os.listdir(path)
        for name in names:
            base_name = os.path.splitext(name)[0]
            full_path = os.path.join(path, name)
            if os.path.isdir(full_path) and recursive:
                packages += self._load_packages(full_path, recursive = recursive)

        return packages

    def _load_modules(self, path, recursive = True):
        modules = []
        names = os.listdir(path)
        for name in names:
            base_name = os.path.splitext(name)[0]
            full_path = os.path.join(path, name)
            if os.path.isdir(full_path) and recursive:
                modules += self._load_modules(full_path, recursive = recursive)
            elif name.endswith(".py"):
                sys.path.insert(0, path)
                try:
                    modules.append(__import__(base_name))
                except ImportError:
                    pass
                finally:
                    sys.path.remove(path)
        return modules



#!/usr/bin/python
# -*- coding: utf-8 -*-

try: from selenium.common.exceptions import ElementClickInterceptedException
except ImportError: ElementClickInterceptedException = None

try: from selenium.common.exceptions import ElementNotVisibleException
except ImportError: ElementNotVisibleException = None

from .. import parts

class InteractionsPart(parts.Part):

    def try_click(self, element):
        try:
            element.click()
            return element
        except (ElementClickInterceptedException, ElementNotVisibleException) as exception:
            self.logger.debug("Element is not \"clickable\" because: %s" % exception)
            return None

    def click_when_possible(self, selector):
        element = self.waits.element(selector)
        return self.waits.until(
            lambda _: self.try_click(element),
            "Element '%s' found but never became clickable" % selector
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

try: from selenium.webdriver.support.ui import WebDriverWait
except ImportError: WebDriverWait = None

from .. import parts

class WaitsPart(parts.Part):

    def __init__(self, owner):
        parts.Part.__init__(self, owner)
        self.wait = WebDriverWait(self.driver, self.timeout)

    def until(self, method, message = None):
        return self.wait.until(
            method,
            message = message
        )

    def redirected_to(self, url):
        return self.until(
            lambda _: url in self.driver.current_url,
            message = "Expecting the page to be '%s' but is '%s', even after waiting '%s' seconds." % (
                url,
                self.driver.current_url,
                self.timeout
            )
        )

    def element(self, selector):
        return self.until(
            lambda _: self.assertions.exists(selector),
            "Element '%s' not found after '%s' seconds." % (selector, self.timeout)
        )

    def elements(self, selector, condition = None):
        return self.until(
            lambda _: self.assertions.exists_multiple(selector, condition = condition),
            "Elements '%s' not found after '%s' seconds." % (selector, self.timeout)
        )

    def text(self, selector, text):
        return self.until(
            lambda _: self.assertions.has_text(selector, text),
            "Element '%s' with text '%s' not found after '%s' seconds." % (selector, text, self.timeout)
        )

    def is_not_visible(self, selector):
        return self.until(
            lambda _: not self.assertions.is_visible(selector),
            "Element '%s' is still visible after '%s' seconds." % (selector, self.timeout)
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import assertions
from . import interactions
from . import waits

from .assertions import AssertionsPart
from .interactions import InteractionsPart
from .waits import WaitsPart



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

try: from selenium.common.exceptions import NoSuchElementException
except ImportError: NoSuchElementException = None

from .. import parts

class AssertionsPart(parts.Part):

    def at_url(self, url):
        current_url = self.driver.current_url

        if url in current_url:
            return True

        self.logger.debug("Current page is '%s' and not '%s'." % (current_url, url))

        return False

    def has_text(self, selector, text):
        selector = appier.legacy.u(selector)
        text = appier.legacy.u(text)

        element = self.exists(selector)

        if element and not element.text == text:
            self.logger.debug("Element '%s' found but has text '%s' instead of '%s'." % (
                selector,
                element.text,
                text
            ))

            return None

        self.logger.debug("Found element '%s' with text '%s'." % (
            selector,
            text
        ))

        return element

    def exists_multiple(self, selector, condition = None):
        elements = self.driver.find_elements_by_css_selector(selector)

        if len(elements) == 0:
            self.logger.debug("Could not find elements with '%s'." % selector)

            return elements

        matching_elements = [element for element in elements if condition(element)] if condition else elements

        if len(matching_elements) == 0:
            self.logger.debug("Found elements with '%s' but none matches the condition." % selector)
        elif condition:
            self.logger.debug("Found elements with '%s' that match the condition." % selector)
        else:
            self.logger.debug("Found elements with '%s'." % selector)

        return matching_elements

    def exists(self, selector):
        try:
            element = self.driver.find_element_by_css_selector(selector)

            self.logger.debug("Found element with '%s'." % selector)

            return element
        except NoSuchElementException:
            self.logger.debug("Could not find element with '%s'." % selector)

            return None

    def is_visible(self, selector):
        element = self.exists(selector)
        return element.is_displayed()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import base
from . import logic

BASE_TUPLES = (
    ("assertions", base.AssertionsPart),
    ("interactions", base.InteractionsPart),
    ("waits", base.WaitsPart)
)

RETAIL_TUPLES = (
    ("admin", logic.AdminPart),
    ("retail", logic.RetailPart)
)

class DomainWrapper(object):

    @classmethod
    def wrap_base(cls, instance):
        for name, cls in BASE_TUPLES:
            part = cls(instance)
            setattr(instance, name, part)

    @classmethod
    def wrap_retail(cls, instance):
        for name, cls in RETAIL_TUPLES:
            part = cls(instance)
            setattr(instance, name, part)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

try: from selenium.webdriver.common.keys import Keys
except ImportError: Keys = None

from .. import parts

class AdminPart(parts.Part):

    def login(self, username, password):
        self.driver.get(self.signin_url)

        form = self.driver.find_element_by_css_selector(".form")
        username_input = form.find_element_by_name("username")
        username_input.send_keys(username)
        password_input = form.find_element_by_name("password")
        password_input.send_keys(password)
        password_input.send_keys(Keys.ENTER)

    def login_and_redirect(self, username, password):
        self.login(username, password)

        self.waits.redirected_to(self.base_url)

    def click_forgot(self):
        self.driver.get(self.signin_url)

        forgot = self.driver.find_element_by_css_selector(".forgot a")
        forgot.click()

    def forgot(self, email):
        self.driver.get(self.recover_url)

        email_input = self.driver.find_element_by_name("identifier")
        email_input.send_keys(email)

        recover_button = self.driver.find_element_by_css_selector(".base")
        recover_button.click()

        self.waits.redirected_to(self.base_url)

    @property
    def base_url(self):
        base_url = appier.conf("BASE_URL", "https://ripe-retail-test.platforme.com")
        return appier.conf("ADMIN_URL", base_url)

    @property
    def admin_url(self):
        return "%s/admin" % self.base_url

    @property
    def signin_url(self):
        return "%s/signin" % self.admin_url

    @property
    def recover_url(self):
        return "%s/recover" % self.admin_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import domain
from . import interactive
from . import decorators
from . import loaders
from . import runners

from .domain import *
from .interactive import *
from .decorators import test
from .loaders import Loader, PathLoader
from .runners import Runner, ConsoleRunner
from .test_cases import TestCase
